/*****
FILE   :test.cpp
PROJECT:BHTest.exe
USES   :<none>
USED BY:cycMain.cpp
CREATED:September 1999 BZ
PURPOSE:Defines various tests to be performed with a BarrettHand
NOTES  :Follow the GENERAL TEST FORMAT to create new tests

*****/

/***** GENERAL TEST FORMAT -- Use this as a template for new tests
int TestX(int testNum){ //Edit 'X' to be the number of this test
	if(testNum<0){
		strcpy(&testName[-testNum][0],"The Test Name"); //Specify test name
		return(10); //Specify the default number of cycles
	}

	//Declare local variables
	int		i, errCt = 0, myLoop;
	char	motor[4];
	FILE	*myFile;
	time_t	cur_time;

	ClearScreen();

	//Test Title
	ShowTitle(testNum);

	CenterText("This is the test description which appears during the test.\n");
	CenterText("Edit this text to describe how to administer the test.\n\n");

	//Reset parameters and initialize motors
	bh.Command("FDEF");
	bh.Command("FSAVE");
	strcpy(motor, "1234");
	bh.InitHand(motor);

	//Prepare the output file
	GetOutFile(&myFile);
	cur_time = time(NULL);
	fprintf(myFile, "\n\n*****\n* %s, %d cycles\n* %s*****\n",
		&testName[testNum][0], testCycles[testNum], ctime(&cur_time));

	//Run the test in a loop
	for(myLoop = 0; myLoop < testCycles[testNum]; myLoop++){
		TestStatus(myLoop+1, testCycles[testNum], errCt); //Update status

		//Execute BarrettHand commands here

		//If an error occurs, update errCt and make a note in the log file 
	}

	//Print a summary to the log file
	fprintf(myFile, "\n\nSUMMARY:\nErrors: %d\n", errCt);
	fclose(myFile);

	//Show summary on screen
	sprintf(myStr, "Test completed with %d errors.", errCt);
	ShowSummary(myStr);

	return(0);
}

//Do not forget to update cycMain.cpp's NUM_TESTS to reflect the number of tests
//Do not forget to update InitializeSoftware()'s test array to include this test

*****/

int Test1(int testNum){

	if(testNum<0){
		strcpy(&testName[-testNum][0],"Finger Movement"); //Specify test name
		return(2); //Return default cycles
	}

	//Declare local variables
	int		fingerSeq[4], i, j, errCt = 0, myLoop;
	bool	isFail;
	char	motor[4], myChar, myStr[80];
	FILE	*myFile;
	time_t	cur_time;

	ClearScreen();

	//Test Title
	ShowTitle(testNum);

	CenterText("I will randomly close and open the fingers and the spread,\n");
	CenterText("you tell me which motors were activated [1, 2, 3, 4, or <N>one].\n\n");

	//Reset parameters
	bh.Command("FDEF");
	bh.Command("FSAVE");
	strcpy(motor, "1234");
	bh.InitHand(motor);

	//bh.Close("S");

	//Prepare the output file
	GetOutFile(&myFile);
	cur_time = time(NULL);
	fprintf(myFile, "\n\n*****\n* Finger Movement, %d cycles\n* %s*****\n",
		testCycles[testNum], ctime(&cur_time));

	srand( (unsigned)time( NULL ) ); //Seed the random number generator

	for(myLoop = 0; myLoop < testCycles[testNum]; myLoop++){
		for( i = 0; i < 4; i++ ){ //Populate the finger sequence array
			do{
				isFail = FALSE;
				fingerSeq[i] = (rand() % 4) + 1;
				for(j = 0; j < i; j++)
					if(fingerSeq[j] == fingerSeq[i]) isFail = TRUE;
			}while(isFail);
		}
		for(i = 0; i < 4; i++){
			TestStatus(myLoop+1, testCycles[testNum], errCt); //Update status
			sprintf(motor, "%d", fingerSeq[i]);

			do{
				isFail = FALSE;
				bh.Close(motor);
				bh.Open(motor);
				MoveCursorTo(7,0);
				printf("\t\t\t\t\t\t\t\t\t\t\t\n");
				MoveCursorTo(7,5);
				printf("\rWhich motor? ");
				myChar = _getch(); _putch(myChar);
				if(myChar - '0' != fingerSeq[i]){
					printf("  <-- You should have seen motor %d move, try again? (Y/N): ", fingerSeq[i]);
					myChar = _getch(); _putch(myChar);
					myChar = toupper(myChar);
					if(myChar == 'N'){ //Log the error
						errCt++;
						printf("\n\n");
						sprintf(myStr, "TEST FAILED: M%d IS NOT FUNCTIONING PROPERLY!", fingerSeq[i]);
						CenterText(myStr); printf("\n");
						CenterText("Please fix it before continuing with these tests."); printf("\n\n");
					
						for(j = testNum+1; j<=NUM_TESTS; j++) testCycles[j] = 0; //Cancel all tests
						fprintf(myFile, "\n%s\n\n", myStr);
						fclose(myFile);

						CenterText("Press any key to return to the Main Menu...");
						myChar = _getch(); //Wait for a response

						return(errCt);

					}else isFail = TRUE;
				}
			}while(isFail);


		}
	}


	sprintf(myStr, "TEST COMPLETED SUCCESSFULLY");

	fprintf(myFile, "\n%s\n\n", myStr);
	fclose(myFile);

	ShowSummary(myStr);

	return(errCt);
}


int Test2(int testNum){
	//This is Test2

	if(testNum<0){
		strcpy(&testName[-testNum][0],"Cable Tension/BreakAway"); //Specify test name
		return(1); //Return default cycles
	}

	int		errCt = 0, go;
	char	motor[4];
	FILE	*myFile;
	time_t	cur_time;

	ClearScreen();

	//Test Title
	ShowTitle(testNum);

	CenterText("1) Cabe Tension: Press any key to close/open the grasp.\n");
	CenterText("The fingers should all move without any obvious motor strain.\n");
	printf("\n");
	CenterText("2) Breakaway: Place your hand across the inner finger links.\n");
	CenterText("I will attempt to close the grasp.  This should cause breakaway\n");
	CenterText("in all fingers. If a finger fails, please adjust the cable tension and/or\n");
	CenterText("manually break the finger away using a 2-mm hex driver.\n");
	CenterText("Press any key to run/continue the test, <ESC> when finished.\n");


	//Reset parameters
	bh.Command("FDEF");
	bh.Command("FSAVE");
	strcpy(motor, "1234");
	bh.InitHand(motor);

	bh.Close("S");
	bh.Command("T");

	strcpy(motor, "G"); //Use full grasp
	go = 1;
	do{
		//TestStatus(myLoop+1, testCycles[2], errCt); //Update status

		if(_getch() == 27)
			go = 0; //Detect <ESC>
		if(go) bh.Close(motor);
		//Test for failure

		if(go) if(_getch() == 27) go = 0; //Detect <ESC>
		if(go) bh.Open(motor);
		
	}while(go);

	//Prepare the output file
	GetOutFile(&myFile);
	cur_time = time(NULL);
	fprintf(myFile, "\n\n*****\n* Cable Tension/BreakAway, %d cycles\n* %s*****\n",
		testCycles[2], ctime(&cur_time));
	fprintf(myFile, "\nTEST COMPLETED\n\n");
	fclose(myFile);

	
	return(0);
}


int Test3(int testNum){
	//This is Test3

	if(testNum<0){
		strcpy(&testName[-testNum][0],"Strain Gages"); //Specify test name
		return(1); //Return default cycles
	}

	int		myLoop, errCt = 0, i, j;
	char	myChar = 0, motor[4];
	DWORD	myTime, tmstart;
	int		strainX = 1, strainY = 1; //Table position
	int		theStrain; //Hand reading
	bool	doContinue;
	int		grid[3][6]; //Grid of readings
	FILE	*myFile;
	time_t	cur_time;


	hConsoleInput = GetStdHandle(STD_INPUT_HANDLE);

	ClearScreen();

	//Test Title
	ShowTitle(testNum);

	printf("\n");
	CenterText("I will prompt you to apply a certain force on a finger.\n");
	CenterText("Please use the force gage to apply that force, then press\n");
	CenterText("<TAB> or <ENTER> to accept the value.  Press <ESC> when finished.\n\n");

	//Show data grid
	printf("\t\t0g\t250g\t500g\t1000g\t1500g\t2000g\n");
	printf("\tF1\n\tF2\n\tF3");

	//Init grid[][]
	for(i = 0; i<3; i++) for(j = 0; j<6; j++) grid[i][j] = 0;

	//Reset parameters
	bh.Command("FDEF");
	bh.Command("FSAVE");
	strcpy(motor, "1234");
	bh.InitHand(motor);


	for(myLoop = 0; myLoop < testCycles[testNum]; myLoop++){
		TestStatus(myLoop+1, testCycles[testNum], errCt); //Update status
		//Apply forces 0g, 250g, 500g, 1000g, 1500g, 2000g
		//Use Real-time loop to show current values
		bh.Command("T"); //Relax spread

		bh.RTSetFlags( "123", 0, 0, 0, 0, 0, 1, 0, 0, 0 );
		bh.RTStart( "123" );
		tmstart = GetTickCount();
		bh.RTUpdate();

		doContinue = TRUE;
		do
		{
			do{
				myTime = GetTickCount() - tmstart;

				if(myTime > 200) //Display strain every 200 milliseconds
				{
					tmstart = GetTickCount();
					theStrain = bh.RTGetStrain( strainY+'0' );
					MoveCursorTo(8+strainY,4+(8*strainX));

					printf("(%6d)", theStrain);
				}
				
				bh.RTUpdate(); //Send & receive the info blocks
			}while(!_kbhit());

			myChar = _getch();

			//Handle Keyboard
			switch(myChar){
			case 9: //<TAB>
				MoveCursorTo(8+strainY,4+(8*strainX));
				printf(" %6d ", theStrain);
				grid[strainY-1][strainX-1] = theStrain;
				strainX++;
				if(strainX > 6){
					strainX = 1;
					strainY++;
					if(strainY > 3) strainY = 1;
				}
				break;

			case 13: //<RETURN>
				MoveCursorTo(8+strainY,4+(8*strainX));
				printf(" %6d ", theStrain);
				grid[strainY-1][strainX-1] = theStrain;
				strainY++;
				if(strainY > 3){
					strainY = 1;
					strainX++;
					if(strainX > 6) strainX = 1;
				}

				break;

			case 27: //ESC
				doContinue = FALSE;

				break;
			default:
				break;
			}

		}while(doContinue);

		bh.RTAbort();


	}

	//Prepare output file
	GetOutFile(&myFile);
	cur_time = time(NULL);
	fprintf(myFile, "\n\n*****\n* Strain Gages, %d cycles\n* %s*****\n",
		testCycles[testNum], ctime(&cur_time));
	fprintf(myFile, "\t0g\t250g\t500g\t1000g\t1500g\t2000g");
	for(i = 0; i < 3; i++){
		fprintf(myFile, "\nF%d", i+1);
		for(j = 0; j < 6; j++) fprintf(myFile, "\t%d", grid[i][j]);
	}
	fprintf(myFile, "\n\n");
	fclose(myFile);

	return(0);
}


int Test4(int testNum){
	//This is Test4

	if(testNum<0){
		strcpy(&testName[-testNum][0],"Vibration Grasp"); //Specify test name
		return(500); //Return default cycles
	}

	int		myLoop, errCt = 0, i, j;
	int		initialValue[4], currentValue[4], worstValue[4];
	char	motor[4], myChar, myStr[80];
	FILE	*myFile;
	time_t	cur_time;
	bool	flip;
	DWORD	myTime, tmstart;

	ClearScreen();

	//Test Title
	ShowTitle(testNum);

	CenterText("Please make sure the hand is mounted securely to the\n");
	CenterText("vibration unit.\n");
	CenterText("Commands: <P>ause, <A>bort\n\n");

	CenterText("Press any key when ready..."); _getch();

	//Select a new serial port?

	//Re-initialize hand, in case of disconnect
	//Reset parameters
	bh.Command("FDEF");
	bh.Command("FSAVE");
	strcpy(motor, "1234");
	bh.InitHand(motor);

	//Close the spread
	bh.Close("S");

	//Set proper motor speeds (65/?)
	bh.Command("FSET MOV 65");
	bh.Command("FSET MCV 65");

	//Select the motors
	strcpy(motor, "G");

	//Get the initial strain gage readings
	bh.Get(motor, "SG", initialValue);

	//Set the worst to the initial values
	for(i = 0; i < 3; i++) worstValue[i] = initialValue[i];

	//Prepare the output file
	GetOutFile(&myFile);
	cur_time = time(NULL);
	fprintf(myFile, "\n\n*****\n* Vibration Grasp, %d cycles\n* %s*****\n",
		testCycles[testNum], ctime(&cur_time));

	for(myLoop = 0; myLoop < testCycles[testNum]; myLoop++){
		TestStatus(myLoop+1, testCycles[testNum], errCt); //Update status
		//Cycle = GC, GO
		bh.Close(motor);
		for(i = 0; i<3; i++){
			errCt += (isErr = CheckFingerClose(i+1, myLoop+1, myFile)); //Check for finger errors on close
			if(isErr) bh.Close("S");
		}
		bh.Open(motor);
		for(i = 0; i<3; i++){
			errCt += (isErr = CheckFingerOpen(i+1, myLoop+1, myFile)); //Check for finger errors on close
			if(isErr) bh.Close("S");
		}

		//Keep track of maximum strain gage drift
		bh.Get(motor, "SG", currentValue);
		for(i = 0; i<3; i++){
			if(abs(currentValue[i]-initialValue[i]) > abs(worstValue[i]-initialValue[i])) 
				worstValue[i] = currentValue[i];
		}

		//Check for keyboard interrupts
		if(_kbhit()){
			myChar = toupper(_getch());
			switch(myChar){
			case 'P': //Pause

				flip = 1;
				tmstart = GetTickCount() - 1000;

				do{
					myTime = GetTickCount() - tmstart;
					if(myTime > 1000){
						tmstart = GetTickCount();
						MoveCursorTo(21,0);
						if(flip) CenterText("*** PAUSED ***");
						else CenterText ("              ");
						flip = !flip;
					}
				}while(!(_kbhit()));

				MoveCursorTo(21,0);
				CenterText ("              ");

				break;

			case 'A': //Abort
				sprintf(myStr, "Test aborted in cycle %d of %d.", myLoop+1, testCycles[testNum]);
				testCycles[testNum] = myLoop+1;
				for(j = testNum+1; j<=NUM_TESTS; j++) testCycles[j] = 0; //Cancel all tests

				//Log results to file
				fprintf(myFile, "\n\n%s", myStr);
				fprintf(myFile, "\n\nSUMMARY:\nErrors: %d\n", errCt);
				fprintf(myFile, "\nMax gage drift\n");
				for(i = 0; i<3; i++) fprintf(myFile, "F%d: %d\n", i+1, worstValue[i] - initialValue[i]);
				fclose(myFile);

				ShowSummary(myStr);

				return(1);

				break;
			}
		}
			



	}

	//Log results to file
	fprintf(myFile, "\n\nSUMMARY:\nErrors: %d\n", errCt);
	fprintf(myFile, "\nMax gage drift\n");
	for(i = 0; i<3; i++) fprintf(myFile, "F%d: %d\n", i+1, worstValue[i] - initialValue[i]);
	fclose(myFile);

	MoveCursorTo(10,0);
	CenterText("Test complete, press any key to continue..."); _getch();

	return(0);

	
}


int Test5(int testNum){
	//This is Test5

	if(testNum<0){
		strcpy(&testName[-testNum][0],"Multi-Axis"); //Specify test name
		return(1500); //Return default cycles
	}

	int		i, j, myLoop, errCt = 0, maxTemp = 0, currentTemp;
	char	spread[4], grasp[4], motor[4], myChar, myStr[80];
	FILE	*myFile;
	time_t	cur_time;
	bool	flip;
	DWORD	tmstart, myTime;

	ClearScreen();

	//Test Title
	ShowTitle(testNum);

	CenterText("I am performing an all-motor cycle test.\n");
	CenterText("Please make sure nothing interferes with the hand.\n");
	CenterText("Commands: <P>ause, <A>bort\n");

	//Reset parameters
	bh.Command("FDEF");
	bh.Command("FSAVE");
	strcpy(motor, "1234");
	bh.InitHand(motor);

	//Set proper motor speeds (150/45)
	bh.Command("FSET MOV 150");
	bh.Command("FSET MCV 150");
	bh.Command("4FSET MOV 45");
	bh.Command("4FSET MCV 45");

	//Prepare the output file
	GetOutFile(&myFile);
	cur_time = time(NULL);
	fprintf(myFile, "\n\n*****\n* Multi-Axis, %d cycles\n* %s*****\n",
		testCycles[testNum], ctime(&cur_time));

	//Select the motors
	strcpy(grasp, "G");
	strcpy(spread, "S");
	for(myLoop = 0; myLoop < testCycles[testNum]; myLoop++){
		TestStatus(myLoop+1, testCycles[testNum], errCt); //Update status
		//Cycle = GC, GO, SC, SO
		bh.Close(grasp); 
		for(i = 0; i<3; i++) errCt += CheckFingerClose(i+1, myLoop, myFile); //Check for finger errors on close
		bh.Open(grasp);
		for(i = 0; i<3; i++) errCt += CheckFingerOpen(i+1, myLoop, myFile); //Check for finger errors on open
		bh.Close(spread);
		errCt += CheckSpreadClose(myLoop, myFile); //Check for spread close error
		bh.Open(spread);
		errCt += CheckSpreadOpen(myLoop, myFile); //Check for spread open error

		//Keep track of the maximum temperature
		bh.Temperature(&currentTemp);
		if(currentTemp > maxTemp) maxTemp = currentTemp;

		//Check for keyboard interrupts
		if(_kbhit()){
			myChar = toupper(_getch());
			switch(myChar){
			case 'P': //Pause

				flip = 1;
				tmstart = GetTickCount() - 1000;

				do{
					myTime = GetTickCount() - tmstart;
					if(myTime > 1000){
						tmstart = GetTickCount();
						MoveCursorTo(21,0);
						if(flip) CenterText("*** PAUSED ***");
						else CenterText ("              ");
						flip = !flip;
					}
				}while(!(_kbhit()));

				MoveCursorTo(21,0);
				CenterText ("              ");

				break;

			case 'A': //Abort
				sprintf(myStr, "Test aborted in cycle %d of %d.", myLoop+1, testCycles[testNum]);
				testCycles[testNum] = myLoop+1;
				for(j = testNum+1; j<=NUM_TESTS; j++) testCycles[j] = 0; //Cancel all tests

				//Log results to file
				fprintf(myFile, "\n\n%s", myStr);
				fprintf(myFile, "\n\nSUMMARY:\nErrors: %d\n", errCt);
				fprintf(myFile, "Maximum temperature: %d\n", maxTemp);
				fclose(myFile);

				ShowSummary(myStr);

				return(1);

				break;
			}
		}

	}
	
	fprintf(myFile, "\n\nSUMMARY:\nErrors: %d\n", errCt);
	fprintf(myFile, "Maximum temperature: %d\n", maxTemp);
	fclose(myFile);

	sprintf(myStr, "Test completed with %d errors.", errCt);
	ShowSummary(myStr);

	return(0);
	
}

int Test6(int testNum){
	//This is Test6

	if(testNum<0){
		strcpy(&testName[-testNum][0],"Encoder slip - Grasp"); //Specify test name
		return(20); //Return default cycles
	}

	int		i, j, myLoop, errCt = 0, initialValue[4], finalValue[4];
	char	motor[4], myStr[80], myChar;
	FILE	*myFile;
	time_t	cur_time;
	bool	flip;
	DWORD	tmstart, myTime;

	ClearScreen();

	//Test Title
	ShowTitle(testNum);

	CenterText("Please make sure the hand is mounted securely to the\n");
	CenterText("bench stand.  I will attempt to slip the finger encoders by\n");
	CenterText("performing motor cycles at high speeds.\n");
	CenterText("Commands: <P>ause, <A>bort\n");

	//Select a new serial port?

	//Reset parameters
	bh.Command("FDEF");
	bh.Command("FSAVE");
	strcpy(motor, "1234");
	bh.InitHand(motor);

	//Set proper motor speeds (255/?)
	bh.Command("FSET MOV 255");
	bh.Command("FSET MCV 255");

	//Select the motors
	strcpy(motor, "G");

	//Read the intial encoder values
	bh.Get("G", "P", initialValue);

	//Prepare the output file
	GetOutFile(&myFile);
	cur_time = time(NULL);
	fprintf(myFile, "\n\n*****\n* Encoder slip - Grasp, %d cycles\n* %s*****\n",
		testCycles[testNum], ctime(&cur_time));

	for(myLoop = 0; myLoop < testCycles[testNum]; myLoop++){
		TestStatus(myLoop+1, testCycles[testNum], errCt); //Update status
		//Cycle = GC, GO
		bh.Close(motor);
		for(i = 0; i<3; i++) errCt += CheckFingerClose(i+1, myLoop+1, myFile); //Check for finger errors on close
		bh.Open(motor);
		for(i = 0; i<3; i++) errCt += CheckFingerOpen(i+1, myLoop+1, myFile); //Check for finger errors on open

		//Check for keyboard interrupts
		if(_kbhit()){
			myChar = toupper(_getch());
			switch(myChar){
			case 'P': //Pause

				flip = 1;
				tmstart = GetTickCount() - 1000;

				do{
					myTime = GetTickCount() - tmstart;
					if(myTime > 1000){
						tmstart = GetTickCount();
						MoveCursorTo(21,0);
						if(flip) CenterText("*** PAUSED ***");
						else CenterText ("              ");
						flip = !flip;
					}
				}while(!(_kbhit()));

				MoveCursorTo(21,0);
				CenterText ("              ");

				break;

			case 'A': //Abort
				sprintf(myStr, "Test aborted in cycle %d of %d.", myLoop+1, testCycles[testNum]);
				testCycles[testNum] = myLoop+1;
				for(j = testNum+1; j<=NUM_TESTS; j++) testCycles[j] = 0; //Cancel all tests

				//Read the final encoder values
				bh.Get("G", "P", finalValue);

				//Log results to file
				fprintf(myFile, "\n\n%s", myStr);
				fprintf(myFile, "\n\nSUMMARY:\nErrors: %d\n", errCt);
				fprintf(myFile, "\n\tInit\tFinal\n");
				for(i = 0; i<3; i++) fprintf(myFile, "M%d\t%d\t%d\n", i+1, initialValue[i], finalValue[i]);
				fclose(myFile);

				ShowSummary(myStr);

				return(1);

				break;
			}
		}
	
	
	}
	//Read the final encoder values
	bh.Get("G", "P", finalValue);

	//Compare and log the results
	fprintf(myFile, "\n\tInit\tFinal\n");
	for(i = 0; i<3; i++) fprintf(myFile, "M%d\t%d\t%d\n", i+1, initialValue[i], finalValue[i]);
	fclose(myFile);

	sprintf(myStr, "Test completed with %d errors.", errCt);
	ShowSummary(myStr);

	return(0);
}

int Test7(int testNum){
	//This is Test7

	if(testNum<0){
		strcpy(&testName[-testNum][0],"Encoder slip - Spread"); //Specify test name
		return(20); //Return default cycles
	}

	int		i, j, myLoop, errCt = 0, initialValue, finalValue;
	char	motor[4], myStr[80], myChar;
	FILE	*myFile;
	time_t	cur_time;
	bool	flip;
	DWORD	tmstart, myTime;

	ClearScreen();

	//Test Title
	ShowTitle(testNum);

	CenterText("Please make sure the hand is mounted securely to the\n");
	CenterText("bench stand.  I will attempt to slip the spread encoder by\n");
	CenterText("performing motor cycles at high speeds.  I will also check\n");
	CenterText("for premature breakaway afterwards.\n");
	CenterText("Commands: <P>ause, <A>bort\n");

	//Reset parameters
	bh.Command("FDEF");
	bh.Command("FSAVE");
	strcpy(motor, "1234");
	bh.InitHand(motor);

	//Set proper motor speeds (?/100)
	bh.Command("FSET MOV 55");
	bh.Command("FSET MCV 55");

	//Select the motors
	strcpy(motor, "S");

	//Read the intial encoder value
	bh.Get("4", "P", &initialValue);


	//Prepare output file
	GetOutFile(&myFile);
	cur_time = time(NULL);
	fprintf(myFile, "\n\n*****\n* Encoder slip - Spread, %d cycles\n* %s*****\n",
		testCycles[testNum], ctime(&cur_time));

	for(myLoop = 0; myLoop < testCycles[testNum]; myLoop++){
		TestStatus(myLoop+1, testCycles[testNum], errCt); //Update status
		//Cycle = SC, SO

		bh.Close(motor);
		errCt += CheckSpreadClose(myLoop+1, myFile); //Check for spread close error
		bh.Open(motor);
		errCt += CheckSpreadOpen(myLoop+1, myFile); //Check for spread open error

		//Check for keyboard interrupts
		if(_kbhit()){
			myChar = toupper(_getch());
			switch(myChar){
			case 'P': //Pause

				flip = 1;
				tmstart = GetTickCount() - 1000;

				do{
					myTime = GetTickCount() - tmstart;
					if(myTime > 1000){
						tmstart = GetTickCount();
						MoveCursorTo(21,0);
						if(flip) CenterText("*** PAUSED ***");
						else CenterText ("              ");
						flip = !flip;
					}
				}while(!(_kbhit()));

				MoveCursorTo(21,0);
				CenterText ("              ");

				break;

			case 'A': //Abort
				sprintf(myStr, "Test aborted in cycle %d of %d.", myLoop+1, testCycles[testNum]);
				testCycles[testNum] = myLoop+1;
				for(j = testNum+1; j<=NUM_TESTS; j++) testCycles[j] = 0; //Cancel all tests

				//Read the final encoder values
				bh.Get("4", "P", &finalValue);

				//Compare and log the results
				fprintf(myFile, "\n\tInit\tFinal\n");
				fprintf(myFile, "M4\t%d\t%d\n", initialValue, finalValue);
				fclose(myFile);

				ShowSummary(myStr);

				return(1);

				break;
			}
		}
	}

	//Read the final encoder values
	bh.Get("4", "P", &finalValue);

	//Compare and log the results
	fprintf(myFile, "\n\tInit\tFinal\n");
	fprintf(myFile, "M4\t%d\t%d\n", initialValue, finalValue);
	
	//Check for Grasp premature breakaway, too
	strcpy(motor, "G");
	bh.Close(motor);
	for(i = 0; i<3; i++) errCt += CheckFingerClose(i+1, 25, myFile); //Check for finger errors on close

	fclose(myFile);

	sprintf(myStr, "Test completed with %d errors.", errCt);
	ShowSummary(myStr);

	return(0);
}

int Test8(int testNum){
	//This is Test8

	if(testNum<0){
		strcpy(&testName[-testNum][0],"Fingerprint"); //Specify test name
		return(1); //Return default cycles
	}

	ClearScreen();

	int		errCt = 0;

	//Test Title
	ShowTitle(testNum);

	CenterText("Please tilt and hold the base stand so that F3 points upward.\n");
	CenterText("First we will check the minimum spread motor velocity, then\n");
	CenterText("I will check encoder accuracy and finger speeds.\n\n");

	CenterText("Press any key when ready...\n\n"); _getch();

	char	motor[4], myStr[80];
	int		value[4], i, j;
	time_t	cur_time;
	FILE	*myFile;


	//Reset parameters
	bh.Command("FDEF");
	bh.Command("FSAVE");
	strcpy(motor, "1234");
	bh.InitHand(motor);

	strcpy(motor, "S");
	//Minimum spread velocity
	for(i = 16; i<=64; i++){
		printf("\r\t\t\tNow trying velocity: %d   ", i); //Update status
		bh.Set(motor, "MCV", i);
		bh.Command("SC");
		bh.Get(motor, "P", value);
		if(value[0] > 3000) i+=63; //Success
		bh.Command("SO");
	}
	GetOutFile(&myFile);
	cur_time = time(NULL);
	fprintf(myFile, "\n\n*****\n* Minimum Spread Velocity\n* %s*****\n", ctime(&cur_time));
	fprintf(myFile, "\nMinimum velocity for sucessful vertical close = %d\n", i-64);
	fclose(myFile);

	MoveCursorTo(11,0);
	CenterText("Thank-you.  Now please return the hand to its normal position.\n\n");
	CenterText("Press any key when ready...\n\n"); _getch();


	long	minHighValue[4], maxHighValue[4], avgHighValue[4];
	long	minLowValue[4], maxLowValue[4], avgLowValue[4];
	
	//Positioning precision/encoder accuracy
	
	strcpy(motor, "G");
	//Perform initial close
	bh.GoToPosition(motor, 16000);
	bh.Get(motor, "P", value);
	for(i = 0; i<3; i++) maxHighValue[i] = avgHighValue[i] = minHighValue[i] = value[i];
	//Perform initial open
	bh.GoToPosition(motor, 50);
	bh.Get(motor, "P", value);
	for(i = 0; i<3; i++) maxLowValue[i] = avgLowValue[i] = minLowValue[i] = value[i];
	for(i = 0; i<24; i++){
		//Perform close
		bh.GoToPosition(motor, 16000);
		bh.Get(motor, "P", value);
		for(j = 0; j<3; j++){
			if(value[j] > maxHighValue[j] ) maxHighValue[j] = value[j];
			if(value[j] < minHighValue[j] ) minHighValue[j] = value[j];
			avgHighValue[j] += value[j];
		}

		//Perform open
		bh.GoToPosition(motor, 50);
		bh.Get(motor, "P", value);
		for(j = 0; j<3; j++){
			if(value[j] > maxLowValue[j] ) maxLowValue[j] = value[j];
			if(value[j] < minLowValue[j] ) minLowValue[j] = value[j];
			avgLowValue[j] += value[j];
		}
	}

	//Calculate the averages
	for(i = 0; i<3; i++){
		avgHighValue[i] /= 25;
		avgLowValue[i] /= 25;
	}

	bh.Command("FDEF");
	bh.Command("FSAVE");

	strcpy(motor, "S");
	//Perform initial close
	bh.GoToPosition(motor, 3000);
	bh.Get(motor, "P", value);
	maxHighValue[3] = avgHighValue[3] = minHighValue[3] = value[0];
	//Perform initial open
	bh.GoToPosition(motor, 30);
	bh.Get(motor, "P", value);
	maxLowValue[3] = avgLowValue[3] = minLowValue[3] = value[0];
	for(i = 0; i<24; i++){
		//Perform close
		bh.GoToPosition(motor, 3000);
		bh.Get(motor, "P", value);
		if(value[0] > maxHighValue[3] ) maxHighValue[3] = value[0];
		if(value[0] < minHighValue[3] ) minHighValue[3] = value[0];
		avgHighValue[3] += value[0];

		//Perform open
		bh.GoToPosition(motor, 30);
		bh.Get(motor, "P", value);
		if(value[0] > maxLowValue[3] ) maxLowValue[3] = value[0];
		if(value[0] < minLowValue[3] ) minLowValue[3] = value[0];
		avgLowValue[3] += value[0];
	}

	//Calculate the average
	avgHighValue[3] /= 25;
	avgLowValue[3] /= 25;


	//Output the result chart
	GetOutFile(&myFile);
	cur_time = time(NULL);
	fprintf(myFile, "\n\n*****\n* Encoder Accuracy (Encoder counts)\n* %s*****\n", ctime(&cur_time));
	fprintf(myFile, "\tMinClse\tAvgClse\tMaxClse\tMinOpen\tAvgOpen\tMaxOpen\n");

	for(i = 0; i<4; i++)
		fprintf(myFile, "M%d\t%ld\t%ld\t%ld\t%ld\t%ld\t%ld\n",
			i+1, minHighValue[i], avgHighValue[i], maxHighValue[i],
			minLowValue[i], avgLowValue[i], maxLowValue[i]);

	fclose(myFile);



	//Finger speeds
	DWORD	myTime, tmstart, fingerTime[3][8];
	int		szbuffer = 10000, N, fingerVel, myFingerVel[3];
	int*	pdata[4][8];  // Pointers to data buffers
	int		go, loop, pos, lastPos, thisPos, myVelocity;
	
	errCt = 0;

	for(i = 0; i<3; i++) for(j = 0; j<8; j++) fingerTime[i][j] = 0;

	//Prepare output file
	GetOutFile(&myFile);
	cur_time = time(NULL);
	fprintf(myFile, "\n\n*****\n* Finger Speeds (seconds)\n* %s*****\n", ctime(&cur_time));

	//Prepare Real-Time variables
	for( int m=0; m<4; m++ )
		for( int v=0; v<8; v++ )
		{
			pdata[m][v] = new int[szbuffer];
			memset((void*)pdata[m][v], 0, szbuffer*sizeof(int));
		}

	bh.RTSetFlags( "123", 1, 1, 0,  1, 1, 0, 1, 1, 1 );
	bh.RTStart( "123" );
	bh.RTUpdate();

	for(loop = 0; loop < 8; loop++){
		switch(loop){
		case 0:
			fingerVel = 20;
			break;
		case 2:
			fingerVel = 55;
			break;
		case 4:
			fingerVel = 75;
			break;
		case 6:
			fingerVel = 100;
			break;
		default:
			fingerVel = -fingerVel;
		}
		for(i = 0; i<3; i++) myFingerVel[i] = fingerVel;
		go = 1;

		N = 0;
		tmstart = GetTickCount();
		while( go ){
			myTime = GetTickCount() - tmstart;

			pos = N%szbuffer;

			for( m=0; m<3; m++ )
			{
				pdata[m][0][pos] = bh.RTGetPosition( m+'1' );
				pdata[m][5][pos] = (int)time;

				if(myTime>100){
					lastPos = pdata[m][0][(N-5)%szbuffer];
					thisPos = pdata[m][0][pos];
					myVelocity = abs((lastPos - thisPos)/10);
					if(myVelocity <= 1){ //Hit the stop
						if(!fingerTime[m][loop]){
							fingerTime[m][loop] = myTime;

							//Check Finger Position
							
							if(myFingerVel[m] > 0){ //Just finished a finger close
								if(thisPos < 17000){ //Failed
									fprintf(myFile, "F%d failed to fully close at V=%d\n", m+1, fingerVel);
									errCt++;
								}
							}else{ //Just finished a finger open
								if(thisPos > 100){ //Failed
									fprintf(myFile, "F%d failed to fully open at V=%d\n", m+1, -fingerVel);
									errCt++;
								}
							}

							myFingerVel[m] = 0;

						}
					}
				}
				bh.RTSetVelocity( m+'1', myFingerVel[m] );

			}
			bh.RTUpdate();
			
			//Check for termination
			go = 0;
			for(i = 0; i<3; i++) go += myFingerVel[i];

			N++;

		}

	}
	bh.RTAbort();

	//Close the hand in preparation for the Freezer
	bh.Close("1234");

	//Output Finger Speed chart
	fprintf(myFile, "\n\n\tClse20\tOpen20\tClse55\tOpen55\tClse75\tOpen75\tClse100\tOpen100");

	for(i = 0; i<3; i++){
		fprintf(myFile, "\nF%d\t", i+1);
		for(j = 0; j<8; j++) fprintf(myFile, "%.3lf\t", fingerTime[i][j] / 1000.0);
	}
	fclose(myFile);

	sprintf(myStr, "Test completed with %d errors.", errCt);
	ShowSummary(myStr);

	//Proper finger interference

	//Proper TSTOP parameter

	//Minimum motor speeds

	//Note strain gage drift




	return(0);
}

int Test9(int testNum){
	//This is Test9

	if(testNum<0){
		strcpy(&testName[-testNum][0],"After-Freezer"); //Specify test name
		return(500); //Return default cycles
	}

	int		i, j, myLoop, errCt = 0, maxTemp = 0, currentTemp;
	char	spread[4], grasp[4], motor[4], myStr[80], myChar;
	FILE	*myFile;
	time_t	cur_time;
	bool	flip;
	DWORD	tmstart, myTime;

	ClearScreen();

	//Test Title
	ShowTitle(testNum);

	CenterText("I am performing an all-motor cycle test.\n");
	CenterText("Please make sure nothing interferes with the hand.\n");
	CenterText("Commands: <P>ause, <A>bort\n");

	//Reset parameters
	bh.Command("FDEF");
	bh.Command("FSAVE");
	strcpy(motor, "1234");
	bh.InitHand(motor);

	//Set proper motor speeds (150/45)
	bh.Command("FSET MOV 150");
	bh.Command("FSET MCV 150");
	bh.Command("4FSET MOV 45");
	bh.Command("4FSET MCV 45");

	//Prepare the output file
	GetOutFile(&myFile);
	cur_time = time(NULL);
	fprintf(myFile, "\n\n*****\n* After Freezer, %d cycles\n* %s*****\n",
		testCycles[testNum], ctime(&cur_time));

	//Select the motors
	strcpy(grasp, "G");
	strcpy(spread, "S");
	for(myLoop = 0; myLoop < testCycles[testNum]; myLoop++){
		TestStatus(myLoop+1, testCycles[testNum], errCt); //Update status
		//Cycle = GC, GO, SC, SO
		bh.Close(grasp); 
		for(i = 0; i<3; i++) errCt += CheckFingerClose(i+1, myLoop, myFile); //Check for finger errors on close
		bh.Open(grasp);
		for(i = 0; i<3; i++) errCt += CheckFingerOpen(i+1, myLoop, myFile); //Check for finger errors on open
		bh.Close(spread);
		errCt += CheckSpreadClose(myLoop, myFile); //Check for spread close error
		bh.Open(spread);
		errCt += CheckSpreadOpen(myLoop, myFile); //Check for spread open error

		//Keep track of the maximum temperature
		bh.Temperature(&currentTemp);
		if(currentTemp > maxTemp) maxTemp = currentTemp;

				//Check for keyboard interrupts
		if(_kbhit()){
			myChar = toupper(_getch());
			switch(myChar){
			case 'P': //Pause

				flip = 1;
				tmstart = GetTickCount() - 1000;

				do{
					myTime = GetTickCount() - tmstart;
					if(myTime > 1000){
						tmstart = GetTickCount();
						MoveCursorTo(21,0);
						if(flip) CenterText("*** PAUSED ***");
						else CenterText ("              ");
						flip = !flip;
					}
				}while(!(_kbhit()));

				MoveCursorTo(21,0);
				CenterText ("              ");

				break;

			case 'A': //Abort
				sprintf(myStr, "Test aborted in cycle %d of %d.", myLoop+1, testCycles[testNum]);
				testCycles[testNum] = myLoop+1;
				for(j = testNum+1; j<=NUM_TESTS; j++) testCycles[j] = 0; //Cancel all tests

				//Log results to file
				fprintf(myFile, "\n\n%s", myStr);
				fprintf(myFile, "\n\nSUMMARY:\nErrors: %d\n", errCt);
				fprintf(myFile, "Maximum temperature: %d\n", maxTemp);
				fclose(myFile);

				ShowSummary(myStr);

				return(1);

				break;
			}
		}

	}
	
	fprintf(myFile, "\n\nSUMMARY:\nErrors: %d\n", errCt);
	fprintf(myFile, "Maximum temperature: %d\n", maxTemp);
	fclose(myFile);

	sprintf(myStr, "Test completed with %d errors.", errCt);
	ShowSummary(myStr);

	return(0);

}


int Test10(int testNum){
	//This is Test10

	if(testNum<0){
		strcpy(&testName[-testNum][0],"Parameter Check"); //Specify test name
		return(1); //Return default cycles
	}

	int		i, j, myLoop, errCt = 0, lineCt, result, dp[4], hp[4]; //Default parameter, Hand parameter
	char	myStr[80], handDesc[1500], handParam[1500], param[8], *myLine, myChar, *myCharPtr;
	FILE	*myFile, *prefs;
	time_t	cur_time;

	ClearScreen();

	//Test Title
	ShowTitle(testNum);

	CenterText("I am verifying the hand parameters.\n");

	//Reset parameters
	//bh.Command("FDEF");
	//bh.Command("FSAVE");

	//Prepare the output file
	GetOutFile(&myFile);
	cur_time = time(NULL);
	fprintf(myFile, "\n\n*****\n* Parameter Check, %d cycles\n* %s*****\n",
		testCycles[testNum], ctime(&cur_time));

	for(myLoop = 0; myLoop < testCycles[testNum]; myLoop++){
		errCt = 0;
		bh.Command("FLISTAV", handParam);
		bh.Command("FLISTA", handDesc);

		//Get the Defaults.dat file
		prefs = fopen("Defaults.dat","r");
		if(!prefs){  //Could not open prefs
			sprintf(myStr, "Could not open preferences file!\n");
			fprintf(myFile, myStr); printf(myStr); //Print to file and screen

			return(1);
		}

		while(fscanf(prefs, "%s %d %d %d %d", param, &dp[0], &dp[1], &dp[2], &dp[3]) != -1){
			myLine = strstr(handDesc, param);
			lineCt = 0;
			if(myLine){ //
				for(myCharPtr = handDesc; myCharPtr != myLine; myCharPtr++) lineCt += (*myCharPtr == '\n');
				for(myLine = handParam; lineCt > 0; myLine++) lineCt -= (*myLine == '\n');
			}

			if(myLine){
				sscanf(myLine, "%d %d %d %d", &hp[0], &hp[1], &hp[2], &hp[3]);
				for(i = 0; i < 4; i++){ //Cycle through all four parameters
					if(dp[i] != hp[i]){ //If the param does not match
						//Resolve it
						sprintf(myStr, "The %s parameter for M%d is %d, but should be %d.", param, i+1, hp[i], dp[i]);
						fprintf(myFile, myStr);
						printf("%s\n", myStr);
						printf("Try to fix (Y/N)? "); myChar = toupper(_getch());
						if(myChar != 'N'){ //Try to fix this parameter directly
							sprintf(myStr, "%d", i + 1);
							bh.Set(myStr, param, dp[i]);
							bh.Command("FSAVE");
							bh.Get(myStr, param, &result);
							if(result == dp[i]){ //Fixed it
								sprintf(myStr, " -Fixed.\n");
								
							}else{ //Can not fix
								sprintf(myStr, " -Can not fix!\n");
								++errCt;
							}
						}else{ //User chose not to fix
							sprintf(myStr, " -User chose not to fix\n");
							++errCt;
						}
						//Print resolution
						fprintf(myFile, myStr); printf(myStr); //Print to file and screen
							
					}
				}
			}else { //If the parameter is not found (?)
				//Note this
				sprintf(myStr, "Parameter '%s' not found in hand!\n", param);
				fprintf(myFile, myStr); printf(myStr); //Print to file and screen
			}
		} //Reached EOF

	}
	fclose(prefs);
	
	fprintf(myFile, "\n\nSUMMARY:\nErrors: %d\n", errCt);
	fclose(myFile);

	sprintf(myStr, "Test completed with %d errors.", errCt);
	ShowSummary(myStr);

	return(0);

}