/*****
FILE   :utils.cpp
PROJECT:BHTest.exe
USES   :<none>
USED BY:cycMain.cpp, test.cpp
CREATED:September 1999 BZ
PURPOSE:Provides several utility routines for BHTest
NOTES  :

*****/

void CenterText(char *theText){
	int len, spaces, i;
	len = strlen(theText);
	spaces = 40 - (len / 2);
	for(i = 0; i<spaces; i++) printf(" ");
	printf(theText);

	return;
}

void MoveCursorTo(int row, int col){
	
	hConsoleOutput = GetStdHandle(STD_OUTPUT_HANDLE);
	myCoord.X = col;
	myCoord.Y = row;

	isErr = SetConsoleCursorPosition(hConsoleOutput,myCoord);

	return;
}

void ClearScreen(void) //Taken from the Visual C++ Help file...
{ 
  COORD coordScreen = { 0, 0 }; // here's where we'll home the cursor 
  BOOL bSuccess; 
  DWORD cCharsWritten; 
  CONSOLE_SCREEN_BUFFER_INFO csbi; // to get buffer info 
  DWORD dwConSize; // number of character cells in the current buffer 
 
  hConsoleOutput = GetStdHandle(STD_OUTPUT_HANDLE);

  //get the number of character cells in the current buffer 
  bSuccess = GetConsoleScreenBufferInfo(hConsoleOutput, &csbi); 
   
  dwConSize = csbi.dwSize.X * csbi.dwSize.Y; 
  //fill the entire screen with blanks 
  bSuccess = FillConsoleOutputCharacter(hConsoleOutput, (TCHAR) ' ', 
      dwConSize, coordScreen, &cCharsWritten); 
  
  //get the current text attribute 
  bSuccess = GetConsoleScreenBufferInfo(hConsoleOutput, &csbi); 
  
  //now set the buffer's attributes accordingly 
  bSuccess = FillConsoleOutputAttribute(hConsoleOutput, csbi.wAttributes, 
      dwConSize, coordScreen, &cCharsWritten); 
  
  //put the cursor at (0, 0)
  bSuccess = SetConsoleCursorPosition(hConsoleOutput, coordScreen); 


  return; 
} 

void WriteInt(unsigned int myInt){
	char myChars[2];

	myChars[0] = myInt & 0x0000FFFF;
	myChars[1] = myInt & 0XFFFF0000;

	bh.ComWrite(myChars, 2);

	return;
}

// Read characters, check for errors
char Read(void)
{
	char buf[10];

	//read character
	if( bh.ComRead( buf, 1 ) )
		return 0;

	return buf[0];
}


// Write characters, check for errors
int Write( char ch )
{
	char buf[2] = " ";
	buf[0] = ch;

	//write character
	return( bh.ComWrite( buf, 1 ) );
		
	//return;
}


// Echo write
int EchoWrite(char ch)
{
	char test;
	int err;

	err = Write( ch );
	
	do{
		test = Read();
	}while( ch != test );

	return(err);
}


int DownloadEEPROM(char *fname){

	int 	num, total_bytes; 
	FILE 	*fhook;
	long 	i;
	float 	per;
	char 	myChar;
	
	//Set the Baud Rate to (Crystal / 4 / 16 / 16); from the M68HC11 Manual
	//We are using a 4.915MHz crystal on the CPU board (not 8MHz, as the example)
	isErr = bh.ComSetBaudrate( 4800 );
	if(isErr) printf("Com4800 Err: %d\n", isErr);

	//Open the EEPROGIX.BOO binary file
	//Found in Motorola's Application Note AN1010 and AS-70
	fhook=fopen("eeprogix.boo","r");
	if(!fhook){
		printf("\nFile not found: EEPROGIX.BOO\n");
		_getch();
		return(1);
	}

	bh.ComClear(); // Clear the communications port
	printf( "\nReset in BOOT mode, press a key to continue..."); _getch();

	//Download the EEPROGIX.BOO binary file
	for(i = 0; !feof(fhook); i++){
		myChar = fgetc(fhook);
		if(i) { // Send character and wait for echo
			isErr = EchoWrite(myChar);
			if(isErr) printf("Com1200 Err: %d\n", isErr);
		}
		else { // Do not wait for echo from first character ($FF trigger byte)
			isErr = Write(myChar);
			if(isErr) printf("Com1200 Err: %d\n", isErr);
		}
		if(!(i%10)) printf("."); // Show download progress
	}
	fclose(fhook);
	printf("\n\nEEPROGIX.BOO downloaded successfully!\n");

	

	//Set the Baud Rate to (Crystal / 4 / 16 / 13); from the M68HC11 Manual
	isErr = bh.ComSetBaudrate( 5908 );
	if(isErr) printf("Com5908 Err: %d\n", isErr);

	//Scan the length of the *.S19 file
	fhook=fopen(fname,"r");
	if(!fhook){
		printf("\nFile not found: %s\n", fname); _getch();
		isErr = bh.ComSetBaudrate( 9600 );
		return(1);
	}else{
		total_bytes = 0;
		while (!feof(fhook)){
			myChar = fgetc(fhook);
			++total_bytes;	
		}
		fclose (fhook);
	}
	
	bh.ComClear(); // Clear the communications port
	EchoWrite('I'); //Write to MC68HC811 <I>nternal EEPROM

	//Download the *.S19 file
	num = 0;
	printf("\nProgress: 0%");
	fhook=fopen(fname,"r");
	while (!feof(fhook)){
		myChar = fgetc(fhook);
		if(feof(fhook)) break;
		EchoWrite(myChar); // Send character and wait for echo
		++num;

		//Update Progress
		per = 100.0 * num / total_bytes;
		printf("\rProgress: %3.0lf%%", per);
	
	}
	fclose (fhook);
	printf("\rProgress: Complete\n");

	printf("\nSet CPU switch 4 to 'OFF' position, press RESET to continue...");

	//Set the Baud Rate to 9600, the BarrettHand will default to this after RESET
	isErr = bh.ComSetBaudrate( 9600 );
	if(isErr) printf("Com9600 Err: %d\n", isErr);

	return(0);
} //End DownloadEEPROM()


int DownloadFirmware(char *fname){
	char stype[3],shex[80], line[100];
	int num,opcode,temp_lo,temp_high,first;
	FILE *fhook;
	int total_bytes;
	long i;
	int count = 0;
	int lobyte, hibyte;
	static int ramarray[300];
	
	count=0;
	total_bytes=0;
	
	//Open the user-specified *.S19 file
	if((fhook=fopen(fname,"r")) == NULL){
		printf("\nFILE NOT FOUND.\n");
		return(1);
	}
	
	//Scan the length of the *.S19 file
	while (!feof(fhook)){
		fgets(line,85,fhook); 
		
		sscanf(line,"%2s%2x%2x%2x%76s",stype,&num,&temp_high,&temp_lo,shex); 
		if (strstr(stype,"S1")) {						 
			num-=3;	    // subtract addr. and checksum from # of pairs 
			total_bytes+=num;
		}
	}
	fclose (fhook);
	
	Write('X'); //Write to MC68HC811 e<X>ternal EEPROM

	printf( "\nPress the red RESET button to begin download..." );
	int errcnt=0;
	while ( Read() != ':' && errcnt<50) errcnt++; //Wait for RESET

	if( errcnt>=50 ){
		printf( "\nDownload Failed\n" );
		return(1);
	}

	Write(255);	 // send first trigger byte 
	
	strcpy(stype,"");
	first=1;
	printf("\nProgress: 0%");

	//Download the *.S19 file
	fhook=fopen(fname,"r");
	while (!feof(fhook)){
		fgets(line,85,fhook); //Read a line of data from *.S19 file
		if( feof(fhook) ) break;

		// basic line format, scan into variables 
		sscanf(line,"%2s%2x%2x%2x%76s",stype,&num,&temp_high,&temp_lo,shex);
		lobyte=temp_lo; hibyte=temp_high;
		if (strstr(stype,"S1")) {  //If this is a DATA S-Record	
			//Update progress				    
			float per;
			num-=3;	    // subtract addr. and checksum from # of pairs 
			per = (100*(double)(count+num)/(double)total_bytes)-1;
			if ( per < 0.0) per = 0.0; 
			count += num;
			printf("\rProgress: %3.0lf%%", per);
			
			for (i=0;i<num; i+=1){  // read pairs into ramarray 
				sscanf(shex+i*2,"%2x",&opcode);
				ramarray[i]=opcode;
			}

			//Pass the data on to Monitor in the BarrettHand
			//Monitor will write the data to external RAM
			Write(255);
			EchoWrite (lobyte);
			EchoWrite (hibyte);
			EchoWrite (num);
			Write(255);
			for (i=0; i<num; i++) {
				EchoWrite (ramarray[i]);
			}
		}
	}
	fclose (fhook);
	printf("\rProgress: 100%\n");

	return(0);
}


void RunBatch(char *fname){
	//Run a batch file
	FILE	*myFile;
	char	myCommand[100], myResult[500];
	int		i, loopCt, len;
	bool	skipCommand;

	if((myFile = fopen(fname,"r")) != NULL){
		while(fgets(myCommand, 99, myFile) != NULL){
			len = strlen(myCommand);
			myCommand[len-1] = '\0';

			skipCommand = FALSE;
			switch(myCommand[0]){
			case '#': //Comment
				printf("%s\n", myCommand);
				skipCommand = TRUE;
				break;
			case 'L': case 'l': //Loop -- Not implemented yet
				for(i = 0; i<loopCt; i++);
			}
			if(!skipCommand){
				//CenterText("Press any key when ready..."); _getch();
				printf("%s\n", myCommand);
				isErr = bh.Command(myCommand, myResult);
				if(myResult[0]!=NULL)  // If command receives hand reply then show it
					printf("-> %s\n", myResult);
				if(isErr){  // If an error is generated
					sprintf(myCommand, "err %d", isErr);
					bh.Command(myCommand, myResult);
					printf("Error %s\n",myResult);
				}
			}
		}
		fclose(myFile);
	}

	return;
}

void GetOutFile(FILE **filePtr){
	char	build_string[100], fileDate[8], thisHand[10], thisCount;
	char	build_prefs[100];
	int		count = 0;
	time_t	cur_time;
	struct	tm *tp;
	FILE	*prefs, *holdFilePtr;

	cur_time = time(NULL);
	tp = localtime(&cur_time);


	//Make the Hand Directory
	//sprintf(build_string, "md %s\n", handSerial);
	//isErr = system(build_string); //Why doesn't this work???  Grrr...

	prefs = fopen("prefs.dat", "r");
	fscanf(prefs, "%s%s%s", handSerial, userInitials, fileDate);

	sprintf(build_prefs, "%s\n%s\n%s\n", handSerial, userInitials, fileDate);
	
	//Search prefs for this S/N
	//If not found, create new x_1.txt, add to prefs
	//If found, if marker or not exists, inc x and new
	//Else append
	
	while(!feof(prefs)){
		fscanf(prefs, "%s%d", thisHand, &thisCount);
		if(feof(prefs)) { //Not found, create new x_1.txt
			thisCount = 1;
			sprintf(build_string, "%s%s%2.2d%2.2d%2.2d_%d.txt",
 				handSerial, userInitials, tp->tm_year, tp->tm_mon + 1, tp->tm_mday, thisCount);
			fclose(prefs);

			prefs = fopen("prefs.dat", "a"); //Append new line to prefs
			fprintf(prefs, "%s\t%d\n", handSerial, thisCount);
			fclose(prefs);
			
			break; 
		}
		

		if(!strcmp(thisHand, handSerial)){ //Found
			sprintf(build_string, "%s%s%2.2d%2.2d%2.2d_%d.txt",
 				handSerial, userInitials, tp->tm_year, tp->tm_mon + 1, tp->tm_mday, thisCount);
			if((thisCount<0) || (!fopen(build_string, "r"))){ //If first write or not there
				int saveCount;
				saveCount = abs(thisCount)+1; //Increment count
				while(!feof(prefs)){
					fscanf(prefs, "%s%d", thisHand, &thisCount);
					if(feof(prefs)) break;
					sprintf(build_prefs, "%s%s\t%d\n", build_prefs, thisHand, thisCount); //Read into memory
				}
				fclose(prefs);

				prefs = fopen("prefs.dat", "w");//Write out old prefs, without this hand
				fprintf(prefs, "%s", build_prefs);
				fclose(prefs);

				prefs = fopen("prefs.dat", "a"); //Append new line to prefs
				fprintf(prefs, "%s\t%d\n", handSerial, saveCount);
				fclose(prefs);

				sprintf(build_string, "%s%s%2.2d%2.2d%2.2d_%d.txt",
 					handSerial, userInitials, tp->tm_year, tp->tm_mon + 1, tp->tm_mday, saveCount);
				
			} //else append

			break;

		}else{
			sprintf(build_prefs, "%s%s\t%d\n", build_prefs, thisHand, thisCount); //Read into memory
		}
	}


/*
	if(fileCreated){
		sprintf(build_string, "%s%s%2.2d%2.2d%2.2d_%d.txt",
 				handSerial, userInitials, tp->tm_year, tp->tm_mon + 1, tp->tm_mday, fileCreated);

	}else{
		// Create Next Test History File
		do{
			count++;
			sprintf(build_string, "%s%s%2.2d%2.2d%2.2d_%d.txt",
 				handSerial, userInitials, tp->tm_year, tp->tm_mon + 1, tp->tm_mday, count);

		}while(fopen(build_string, "r"));
		fileCreated = count;
	}
*/
	*filePtr = fopen(build_string, "a");

	return;
}


#define JAMMED_OPEN		3000
#define JAMMED_CLOSE	500
#define PRE_BREAK		10000
#define NO_BREAK		15100
#define SPREAD_OPEN		50
#define SPREAD_CLOSED	3050

int CheckFingerClose(int finger, int cycle, FILE *myFile)
{
	int		value, myErr = 0, restoreValue, result;
	char	motor[4];
	time_t	cur_time;

	sprintf(motor, "%d", finger);
	bh.Get(motor, "P", &value);

	if(value < JAMMED_OPEN){ // IF JAMMED OPEN
		myErr = 1;

		bh.Get(motor, "MOV", &restoreValue);
		result=bh.Set(motor, "MOV", 65); //Set the Finger Open speed to a 'normal' value
		result=bh.InitHand( motor );
		result=bh.Set(motor, "MOV", restoreValue);


		//Log the error
		cur_time = time(NULL);
		fprintf(myFile, "F%d JAMMED OPEN in cycle %d at %s", finger, cycle, ctime(&cur_time));

	}
	else if(value < PRE_BREAK){ // IF PREMATURE BREAKAWAY
		myErr = 1;

		bh.Get(motor, "MOV", &restoreValue);
		result=bh.Set(motor, "MOV", 65); //Set the Finger Open speed to a 'normal' value
		result=bh.InitHand( motor );
		result=bh.Set(motor, "MOV", restoreValue);

		//Log the error
		cur_time = time(NULL);
		fprintf(myFile, "F%d PREMATURE BREAKAWAY in cycle %d at %s", finger, cycle, ctime(&cur_time));
	}
	else if(value < NO_BREAK){ // IF NO BREAKAWAY
		myErr = 1;

		//Don't initialize, this is a common failure
		//sprintf(motor, "%d", i+1);
		//result=bh.InitHand( motor );

		//Log the error
		cur_time = time(NULL);
		fprintf(myFile, "F%d NO BREAKAWAY in cycle %d at %s", finger, cycle, ctime(&cur_time));


	}
	else{
		// Success!
	}
	
	return(myErr);
}


int CheckFingerOpen(int finger, int cycle, FILE *myFile)
{
	int		myErr = 0, value, restoreValue, result;
	char	motor[4];
	time_t	cur_time;

	sprintf(motor, "%d", finger);
	bh.Get(motor, "P", &value);

	if(value > JAMMED_CLOSE){ // IF JAMMED CLOSE
		myErr = 1;

		bh.Get(motor, "MOV", &restoreValue);
		result=bh.Set(motor, "MOV", 65); //Set the Finger Open speed to a 'normal' value
		result=bh.InitHand( motor );
		result=bh.Set(motor, "MOV", restoreValue);

		//Log the error
		cur_time = time(NULL);
		fprintf(myFile, "F%d JAMMED CLOSE in cycle %d at %s", finger, cycle, ctime(&cur_time));


	}
	else{
		// Success!
	}

	return(myErr);
}


int CheckSpreadClose(int cycle, FILE *myFile)
{
	int		myErr = 0, value, restoreValue, result;
	char	motor[4];
	time_t	cur_time;

	strcpy(motor, "S");
	bh.Get(motor, "P", &value);

	if(value < SPREAD_CLOSED){ // IF SPREAD DID NOT FULLY CLOSE
		myErr = 1;

		bh.Get(motor, "MOV", &restoreValue);
		result=bh.Set(motor, "MOV", 35); //Set the Spread Open speed to a 'normal' value
		result=bh.InitHand( motor );
		bh.Set(motor, "MOV", restoreValue);

		//Log the error
		cur_time = time(NULL);
		fprintf(myFile, "Spread DID NOT FULLY CLOSE in cycle %d at %s", cycle, ctime(&cur_time));


	}
	else{
		// Success!
	}

	return(myErr);
}

int CheckSpreadOpen(int cycle, FILE *myFile)
{
	int		myErr = 0, value, restoreValue, result;
	char	motor[4];
	time_t	cur_time;

	strcpy(motor, "S");
	bh.Get(motor, "P", &value);

	if(value > SPREAD_OPEN){ // IF SPREAD DID NOT FULLY OPEN
		myErr = 1;

		bh.Get(motor, "MOV", &restoreValue);
		result=bh.Set(motor, "MOV", 35); //Set the Spread Open speed to a 'normal' value
		result=bh.InitHand( motor );
		bh.Set(motor, "MOV", restoreValue);

		//Log the error
		cur_time = time(NULL);
		fprintf(myFile, "Spread DID NOT FULLY OPEN in cycle %d at %s", cycle, ctime(&cur_time));


	}
	else{
		// Success!
	}

	return(myErr);
}

void TestStatus(int cycle, int total, int err){
	char	myStr[80];

	sprintf(myStr, "Cycle %d of %d, Errors = %d\n\n", cycle, total, err);
	MoveCursorTo(1,0);
	CenterText(myStr);

	return;
}

void ShowTitle(int num){

	//Test Title
	MoveCursorTo(0,0);
	CenterText(&testName[num][0]);
	TestStatus(1, testCycles[num], 0); //Update status
}

void ShowSummary(char *mySummary){
	DWORD	myTime, tmstart;
	int		mySeconds = 10;
	char	myStr[40];

	MoveCursorTo(20,0);
	CenterText(mySummary);
	MoveCursorTo(21,0);

	tmstart = GetTickCount() - 1000;

	do{
		myTime = GetTickCount() - tmstart;
		if(myTime > 1000){
			tmstart = GetTickCount();
			printf("\r");
			sprintf(myStr, "Press any key to continue [%2.2d]...", mySeconds);
			CenterText(myStr);
			--mySeconds;
		}
	}while(!(_kbhit()) && mySeconds);

	return;
}