/*****
FILE   :cycMain.cpp
PROJECT:BHTest.exe
USES:	test.cpp	-- source for BarrettHand tests
		utils.cpp	-- utilities for program operation
USED BY:<none>
CREATED:September 1999 BZ
PURPOSE:Provides the ability to cycle, to test, and to checkout
		a BarrettHand.
NOTES  :

*****/


#include <stdio.h> //Already in BHand.cpp
#include <conio.h>
#include <time.h>
#include <stdlib.h>
#include <windows.h> //Already in BHand.h

/* Define "bool" for older compilers 

#define TRUE 1
#define FALSE 0
typedef char bool;

*/

//#include "BHand.cpp"
#include "BHand.h"

//Define the number of tests in the application
//If another test is added to test.cpp, this number must be updated!
#define NUM_TESTS 10 

//Declare GLOBAL variables
int		isErr, hasGages, doQuit = 0, testCycles[NUM_TESTS+1], fileCreated = 0;
char	handSerial[10], userInitials[10], testName[20][40], fieldStr[10], *fieldStrPtr;
int		(*doTest[NUM_TESTS+1])(int cycles);
HANDLE	hConsoleInput, hConsoleOutput;
COORD	myCoord;
BHand	bh;

#include "Utils.cpp"
#include "Test.cpp"

//LoadDefaults() -- Initial read/creation of the preferences file
void LoadDefaults(void){
	FILE *prefs;
	char	build_string[100], prefsDate[8], thisCount, thisHand[10];
	time_t	cur_time;
	struct	tm *tp;

	//Build today's date string, ex: "990915"
	cur_time = time(NULL);
	tp = localtime(&cur_time);
	sprintf(build_string, "%2.2d%2.2d%2.2d", tp->tm_year, tp->tm_mon + 1, tp->tm_mday);

	//Load the default cycle values
	//Get the default value by calling the function with cycle value = -(testNum)
	for(int i=1;i<=NUM_TESTS;i++) testCycles[i] = doTest[i](-i);

	//Read in the last handSerial and userInitials from the prefs file
	if((prefs = fopen("Prefs.dat","r"))==NULL){  // prefs file must be created
		prefs = fopen("Prefs.dat","w");
		fprintf(prefs, "HND000\n");
		fprintf(prefs, "--\n");
		fprintf(prefs, "%s\n", build_string);
		fclose(prefs);
	}
	prefs = fopen("Prefs.dat","r");
	fscanf(prefs, "%s%s%s", handSerial, userInitials, prefsDate);

	//Check prefs date
	if(strcmp(prefsDate, build_string)){ //If not today, delete rest of prefs
		fclose(prefs);
		prefs = fopen("Prefs.dat","w");
		fprintf(prefs, "%s\n%s\n%s\n", handSerial, userInitials, build_string);
	}
	else{ //If today, set markers for each hand
		sprintf(build_string, "%s\n%s\n%s\n", handSerial, userInitials, prefsDate);
		while(!feof(prefs)){
			
			fscanf(prefs, "%s%d", thisHand, &thisCount);
			if(feof(prefs)) break;
			sprintf(build_string, "%s%s\t%d\n", build_string, thisHand, -abs(thisCount));
		}
		fclose(prefs);
		prefs = fopen("Prefs.dat","w");
		fprintf(prefs, "%s", build_string);
	}
	
	fclose(prefs);

	return;
}

void InitializeSoftware(void){
	//Select a com port (user-specified)
	char myChar = 0;

	//Initialize the software & Com port and set up a BarrettHand handle
	printf("Please select a communications port [1]: ");
	myChar = getchar();
	if(myChar == 10) myChar = 1;
	else myChar -= '0';

	isErr = bh.InitSoftware(myChar);

	if(isErr){
		printf("\n\nInitialization error: %d\n", isErr); 
		switch(isErr){ //Check for common errors
		case -2:
			printf("Communications port not available...");
			break;
		case -9:
			printf("BarrettHand not found!");
			break;
		default:
			break;
		}
		
		printf("\n\nContinue (Y/N)? ");
		myChar = toupper(_getch()); //Wait for acknowledgment
		if(myChar == 'N') exit(1);
	}

	//Set up array of test procedures
	//If another test is added to test.cpp, add it to this list-
	//Example: doTest[11] = Test11;
	//If a different order is desired, just change the right-most column
	//Example: doTest[1] = Test2;
	//		   doTest[2] = Test1;
	doTest[1] = Test1;
	doTest[2] = Test2;
	doTest[3] = Test3;
	doTest[4] = Test4;
	doTest[5] = Test5;
	doTest[6] = Test6;
	doTest[7] = Test7;
	doTest[8] = Test8;
	doTest[9] = Test9;
	doTest[10] = Test10;

	LoadDefaults();

	return;
}

void SenseHand(void){
	//Check to see if a hand is connected
	//Verify Firmware
	//Check for Strain Gages
	//Would be nice to auto-sense S/N here...

	/*
	char motor[4];

	strcpy(motor, "1234");
	bh.InitHand(motor);

	*/

	if(isErr){
		//Output the error
	}

	return;
}

//handleTab() -- The active field was changed, update the data
void handleTab(int fromField, int toField){
	int		len, i, myValue;
	FILE	*prefs;
	char	build_prefs[100], fileDate[10], thisHand[10], thisCount;

	switch(fromField){
	case 0: //First time only
		break;

	case 1: //Hand Serial Number field
		len = strlen(fieldStr);
		MoveCursorTo(2,20);
		for(i = 0;i<8-len;i++) printf(" "); //Pad it with spaces
		printf("%s", fieldStr);

		//Update Prefs.dat with this hand S/N
		prefs = fopen("prefs.dat", "r");
		fscanf(prefs, "%s%s%s", handSerial, userInitials, fileDate);
		strcpy(handSerial,fieldStr);
		sprintf(build_prefs, "%s\n%s\n%s\n", handSerial, userInitials, fileDate);
		while(!feof(prefs)){
			fscanf(prefs, "%s%d", thisHand, &thisCount);
			if(feof(prefs)) break;
			sprintf(build_prefs, "%s%s\t%d\n", build_prefs, thisHand, thisCount); //Read into memory
		}
		fclose(prefs);

		prefs = fopen("prefs.dat", "w");//Write out new prefs
		fprintf(prefs, "%s", build_prefs);
		fclose(prefs);

		break;

	case 2: //User Initials field
		len = strlen(fieldStr);
		MoveCursorTo(3,20);
		for(i = 0;i<8-len;i++) printf(" ");
		printf("%s", fieldStr);

		//Update Prefs.dat with new user
		prefs = fopen("prefs.dat", "r");
		fscanf(prefs, "%s%s%s", handSerial, userInitials, fileDate);
		strcpy(userInitials,fieldStr);
		sprintf(build_prefs, "%s\n%s\n%s\n", handSerial, userInitials, fileDate);
		while(!feof(prefs)){
			fscanf(prefs, "%s%d", thisHand, &thisCount);
			if(feof(prefs)) break;
			sprintf(build_prefs, "%s%s\t%d\n", build_prefs, thisHand, thisCount); //Read into memory
		}
		fclose(prefs);

		prefs = fopen("prefs.dat", "w");//Write out new prefs
		fprintf(prefs, "%s", build_prefs);
		fclose(prefs);

		break;

	default: //Handle cycle count parameter field
		len = strlen(fieldStr);
		//for(myValue = i = 0; i<len; i++) myValue = (myValue * 10) + (fieldStr[i] - 48);
		sscanf(fieldStr, "%d", &myValue);
		testCycles[fromField - 2] = myValue;
		MoveCursorTo(fromField+4,9);
		for(i = 0;i<8-len;i++) printf(" "); //Pad it with spaces
		printf("%s", fieldStr);

		break;
	}

	fieldStrPtr = fieldStr; //Reset to beginning of string

	switch(toField){
	case 1: //Hand Serial Number field
		strcpy(fieldStr,handSerial);
		len = strlen(fieldStr);
		MoveCursorTo(2,20);
		printf("%s", fieldStr);
		for(i = 0;i<8-len;i++) printf(" ");
		MoveCursorTo(2,20);
		
		break;

	case 2: //User Initials field
		strcpy(fieldStr,userInitials);
		len = strlen(fieldStr);
		MoveCursorTo(3,20);
		printf("%s", fieldStr);
		for(i = 0;i<8-len;i++) printf(" ");
		MoveCursorTo(3,20);

		break;

	default: //Cycle count parameter field
		sprintf(fieldStr, "%d", testCycles[toField - 2]);
		len = strlen(fieldStr);
		MoveCursorTo(toField+4,9);
		printf("%s", fieldStr);
		for(i = 0;i<8-len;i++) printf(" ");
		MoveCursorTo(toField+4,9);

		break;
	}

	return;
}

//myGetKey() -- Keyboard strobe for catching <SHIFT> key
char myGetKey(void){
	char			myChar = 0;
	unsigned long	isShifted;
	INPUT_RECORD	myRec;
	DWORD			actualRead;

	hConsoleInput = GetStdHandle(STD_INPUT_HANDLE);
	do{
		isErr = ReadConsoleInput(hConsoleInput,&myRec,1,&actualRead);
		if((myRec.EventType == KEY_EVENT) && (myRec.Event.KeyEvent.bKeyDown)){
			myChar = myRec.Event.KeyEvent.uChar.AsciiChar;
			isShifted = (myRec.Event.KeyEvent.dwControlKeyState & SHIFT_PRESSED);

			if((myChar == 9) && (isShifted)) myChar = 10; //cheat, 10 = shift tab
			if((myChar == 13) && (isShifted)) myChar = 14; //cheat, 14 = shift return
			
		}

	}while(!myChar);

	return(myChar);
}

//UpdateHandState() -- Show the current motor parameters in the terminal window
void UpdateHandState(void){
	//Need a GetCursorCoordXY() function, know one?

	/*
	int result, value[4];
	char motor[4];

	strcpy(motor, "");

	//Show Header
	MoveCursorTo(0,0);
	printf("TERMINAL MODE - Type 'X' to eXit.");
	MoveCursorTo(0,40);
	printf("S/N: %s", handSerial);
	MoveCursorTo(1,40);
	printf("\tM1\tM2\tM3\tM4");

	//Show Motor Positions
	MoveCursorTo(2,40);
	result=bh.Get( motor , "P", value );
	printf("POS:\t%d\t%d\t%d\t%d", value[0], value[1], value[2], value[3]);

	//Show Strain Gage Values
	MoveCursorTo(3,40);
	result=bh.Get( motor , "S", value );
	printf("STR:\t%d\t%d\t%d\t%d", value[0], value[1], value[2], value[3]);

	//Show MOV
	MoveCursorTo(4,40);
	result=bh.Get( motor , "MOV", value );
	printf("MOV:\t%d\t%d\t%d\t%d", value[0], value[1], value[2], value[3]);

	//Show MCV
	MoveCursorTo(5,40);
	result=bh.Get( motor , "MCV", value );
	printf("MCV:\t%d\t%d\t%d\t%d", value[0], value[1], value[2], value[3]);

	*/

	return;
}

void doTerminal(void){
	char	handCommand[20], handReply[1500], fname[50];
	int		i, exitTerminal = 0, skipCommand, len;

	ClearScreen();
	UpdateHandState();

	//Show Header
	printf("TERMINAL MODE - Type 'X' to eXit.\t\t\t");
	printf("S/N: %s\n\n", handSerial);

	do
	{
		skipCommand = 0;
		printf(">> ");
		gets(handCommand);
		len = strlen(handCommand);
		for(i = 0; i < len; i++) handCommand[i] = toupper(handCommand[i]);

		if(!(strcmp(handCommand, "X"))) exitTerminal = 1;
		if(!(strcmp(handCommand, "BATCH"))){
			skipCommand = 1;
			printf("Run Batch File: ");
			gets(fname);
			RunBatch(fname);
			//Check each command issued for success
		}
		if(!(strcmp(handCommand, "EEPROM"))){
			//skipCommand = 1;
			printf("Load EEPROM: ");
			gets(fname);
			DownloadEEPROM(fname);
			strcpy(handCommand, "RESET");
		}
		if(!(strcmp(handCommand, "FIRMWARE"))){
			skipCommand = 1;
			printf("Load Firmware: ");
			gets(fname);
			DownloadFirmware(fname);

		}
		
		if((!exitTerminal) && (!skipCommand)){
			isErr = bh.Command(handCommand,handReply); // send command to hand
			if(handReply[0]!=NULL)  // If command receives hand reply then show it
				printf("-> %s\n", handReply);
			if(isErr){  // If an error is generated
				sprintf(handCommand, "err %d", isErr);
				bh.Command(handCommand,handReply);
				printf("Error %s\n",handReply);
			}
		}

		UpdateHandState();

	
	}
	while(!exitTerminal);

	return;
}

void DrawMainMenu(void){
	int		len, i;
	time_t	cur_time;

	//Clear the screen
	ClearScreen();

	//Print Title
	printf("BarrettHand Test Application v3.0");

	//Print Current Date & Time
	time(&cur_time);
	printf("\t\t  %s\n", ctime(&cur_time));

	//Draw the fields
	printf("Hand Serial Number [");
	len = strlen(handSerial);
	MoveCursorTo(2,20);
	for(i = 0;i<8-len;i++) printf(" ");
	printf("%s]\n", handSerial);

	printf("User Initials      [");
	len = strlen(userInitials);
	MoveCursorTo(3,20);
	for(i = 0;i<8-len;i++) printf(" ");
	printf("%s]\n", userInitials);

	printf("\n\tCycles\t\tTest\n");
	printf("\t------\t\t----\n");
	for(i = 1; i<=NUM_TESTS; i++) printf("\t[%8d]\t%s\n", testCycles[i], &testName[i][0]);

	//Print help info
	printf("\n\n");
	CenterText("<TAB>, <SHIFT><TAB> Selects field  <CTRL><T> Terminal mode\n");
	CenterText("<CTRL><R> Runs selected test  <CTRL><A> Runs all tests  <CTRL><Q> Quit");

	return;
}

void RunTests(void){
	//Perform the cycle tests

	for(int i = 1; i<=NUM_TESTS; i++) if(testCycles[i]) isErr = doTest[i](i);

	return;
}

void flushBuffers(void){
	hConsoleInput = GetStdHandle(STD_INPUT_HANDLE);
	bool	bSuccess;

	bSuccess = FlushConsoleInputBuffer(hConsoleInput);
	return;
}

void MainMenu(void){
	char myChar, fieldNum = 1, nextField;

	_fcloseall(); //Close any open files

	//Show the Main Menu and allow the user to configure cycle tests
	DrawMainMenu();

	handleTab(0,1); //Initial field

	do{
		myChar = myGetKey();
		switch(myChar){
		case 17: //<CTRL><Q> (Quit)
			flushBuffers(); //Eat the character
			handleTab(fieldNum,fieldNum);
			MoveCursorTo(22,0);
			doQuit = 1;
			break;
			
		case 20: //<CTRL><T> (Terminal mode)
			flushBuffers(); //Eat the character
			handleTab(fieldNum,fieldNum);
			doTerminal();
			DrawMainMenu();
			handleTab(fieldNum,fieldNum);
			break;

		case 18: //<CTRL><R> (Run current test only)
			flushBuffers(); //Eat the character
			if(fieldNum>2){ //Only if they are in a test field
				handleTab(fieldNum,fieldNum);
				if(testCycles[fieldNum-2]) isErr = doTest[fieldNum-2](fieldNum-2);
				DrawMainMenu();
				handleTab(fieldNum,fieldNum);
			}
			break;

		case 1: //<CTRL><A> (Run all tests)
			flushBuffers(); //Eat the character
			handleTab(fieldNum,fieldNum);
			RunTests();
			DrawMainMenu();
			handleTab(fieldNum,fieldNum);
			break;

		case 9: case 13://TAB or RETURN
			if(fieldNum == NUM_TESTS + 2) nextField = 1;
			else nextField = fieldNum + 1;
			handleTab(fieldNum, nextField);
			fieldNum = nextField;

			break;

		case 10: //SHIFT-TAB
			if(fieldNum == 1) nextField = NUM_TESTS + 2;
			else nextField = fieldNum - 1;
			handleTab(fieldNum, nextField);
			fieldNum = nextField;

			break;
			
		case 8: //Backspace (?)
			if(fieldStrPtr - fieldStr > 0){
				printf("%c", myChar);
				--fieldStrPtr;
			}
			break;

		default:
			//printf("%c = %d\n", myChar, myChar);
			if(fieldStrPtr - fieldStr == 8) break; //If field is full

			if(fieldNum > 2){ //If this is a numerical field
				if((myChar >= '0') && (myChar <= '9')){ //Only allow numbers
					printf("%c", myChar);
					sprintf(fieldStrPtr, "%c", myChar);
					if(!(fieldStrPtr++ - fieldStr)){ //If this is the first char in string
						printf("       "); //Clear the rest of the field
						MoveCursorTo(fieldNum+4, 10);
					}
				}
			}
			else{ //This is an alphanumeric field
				myChar = toupper(myChar);
				printf("%c", myChar);
				sprintf(fieldStrPtr, "%c", myChar);
				if(!(fieldStrPtr++ - fieldStr)){ //If this is the first char in string
					printf("       "); //Clear the rest of the field
					MoveCursorTo(fieldNum+1, 21);
				}
			}
		}

	}while(!doQuit);

	return;
}


int main(void){
	InitializeSoftware();

	SenseHand();

	MainMenu();

	return(0);
}

/* Simple Date/Time Stamp Usage Reference
	#include <time.h>
	time_t cur_time;

	time(&cur_time);
	printf("%s", ctime(&cur_time));

   Time Structure Usage
	struct tm *tp;
	tp = localtime(&cur_time);

	printf("%d", tp->tm_sec);

	tm_sec = Seconds after minute (0 - 59)
	tm_min = Minutes after hour (0 - 59)
	tm_hour = Hours after midnight (0 - 23)
	tm_mday = Day of month (1 - 31)
	tm_mon = Month (0 - 11; January = 0)
	tm_year = Year (current year minus 1900)
	tm_wday = Day of week (0 - 6; Sunday = 0)
	tm_yday = Day of year (0 - 365; January 1 = 0)
*/



